# Быстрый старт

## 1. Установка

```bash
opm install benchmark
```

## 2. Создание бенчмарка

Создайте класс с методами, помеченными аннотацией `&Бенчмарк`, где каждый метод реализует тестируемую логику. 
В следующем примере мы сравниваем алгоритмы конкатенации строк через оператор `+` и метод `СтрСоединить`:

```bsl
// КонкатенацияСтрок.os

&Параметры(100, 1000)
Перем Количество Экспорт;
Перем Подстрока;

&МониторингПамяти
Процедура ПриСозданииОбъекта()
	Подстрока = "Просто текст";
КонецПроцедуры

&Бенчмарк
&Эталон
Процедура ОператорСложения() Экспорт
	Текст = "";
	Для НомерПодстроки = 1 По Количество Цикл
		Текст = Текст + Подстрока;
	КонецЦикла;
КонецПроцедуры

&Бенчмарк
Процедура МассивПодстрок() Экспорт
	МассивПодстрок = Новый Массив();
	Для НомерПодстроки = 1 По Количество Цикл
		МассивПодстрок.Добавить(Подстрока);
	КонецЦикла;

	Текст = СтрСоединить(МассивПодстрок);
КонецПроцедуры
```
## 3. Запуск бенчмарков

Запустите бенчмарки одним из следующих способов:

**CLI:**

```bash
benchos run [FILE]
```

`FILE` - имя файла класса бенчмарков в текущем каталоге, либо полный путь к нему

**API:**

```bsl
#Использовать benchmark
#Использовать "."

Результат = Бенчмаркинг.Запустить(Тип("КонкатенацияСтрок"));
```

## 4. Просмотр результатов

После завершения тестов в консоли будет отображена подробная сводная таблица, оформленная в формате Markdown:

```
BenchmarkOneScript v0.3.0, OneScript v2.0.0.567-rc7-29039316, Microsoft Windows NT 10.0.20348.0 \
Intel Core i7-8700 CPU 3.20GHz (Max: 3.19GHz) (Coffee Lake), 1 CPU, 12 logical and 6 physical cores

| Method           | Количество | Baseline |        Mean |      StdErr |      StdDev | Ratio |      Median |     Op/s |    Allocated |
|------------------|-----------:|----------|------------:|------------:|------------:|------:|------------:|---------:|-------------:|
| МассивПодстрок   |        100 | false    |    71.10 us |    218.1 ns |    844.6 ns | 1.559 |    70.79 us | 14,065.4 |     17.77 KB |
| ОператорСложения |        100 | true     |    45.65 us |    348.4 ns |  1,349.3 ns | 1.000 |    45.40 us | 21,907.3 |    126.54 KB |
| МассивПодстрок   |      1,000 | false    |   701.28 us | 13,839.6 ns | 53,600.7 ns | 0.561 |   678.78 us |  1,426.0 |    165.43 KB |
| ОператорСложения |      1,000 | true     | 1,251.74 us | 15,190.6 ns | 58,832.8 ns | 1.000 | 1,236.15 us |    798.9 | 11,808.96 KB |

// Легенда
Количество : Значение параметра 'Количество'
Mean       : Арифметическое среднее всех измерений
StdErr     : Стандартная ошибка всех измерений
StdDev     : Стандартное отклонение всех измерений
Ratio      : Среднее коэффициентов отношений ([Mean] / [Baseline Mean])
Median     : Значение, разделяющее упорядоченные измерения на две равные части
Op/s       : Операций в секунду
Allocated  : Выделяемая память на одну операцию
1 us       : 1 Микросекунда
1 ns       : 1 Наносекунда
```
