# Выбор стратегии запуска

## ПропускнаяСпособность

Стратегия используется по умолчанию. Оптимальна для микробенчмаркинга. 
Автоматическии подбирает количество вызовов за итерацию. Выполняет прогревочные итерации для стабилизации результатов.

### Аннотация

```bsl
&СтратегияПропускнаяСпособность
Процедура ПриСозданииОбъекта()
```

### API

```bsl
Конфигурация.УстановитьСтратегию(СтратегииЗапускаБенчмарка.ПропускнаяСпособность)
```

## ХолодныйЗапуск

Стратегия для оценки производительности при первом запуске без предварительной оптимизации (Отключает оценочные и прогревочные итерации).

### Аннотация

```bsl
&СтратегияХолодныйЗапуск
Процедура ПриСозданииОбъекта()
```

### API

```bsl
Конфигурация.УстановитьСтратегию(СтратегииЗапускаБенчмарка.ХолодныйЗапуск)
```

### Пример

```bsl
Перем ЭтоПервыйВызов;

&СтратегияХолодныйЗапуск
&КоличествоИтераций(5)
&КолонкаМин
&КолонкаМакс
Процедура ПриСозданииОбъекта()
	ЭтоПервыйВызов = Истина;	
КонецПроцедуры

&Бенчмарк
Процедура Бенчмарк() Экспорт
	
	Если ЭтоПервыйВызов Тогда
		ЭтоПервыйВызов = Ложь;
		Приостановить(1000);
	Иначе
		Приостановить(10);
	КонецЕсли;

КонецПроцедуры
```

### Вывод

```md
// Бенчмарк: МойКласс.Бенчмарк

Измерение  1 : 1 op, 1.001 s, 1.001 s/op
Измерение  2 : 1 op, 10.34 ms, 10.34 ms/op
Измерение  3 : 1 op, 10.48 ms, 10.48 ms/op
Измерение  4 : 1 op, 10.19 ms, 10.19 ms/op
Измерение  5 : 1 op, 10.67 ms, 10.67 ms/op

// Сводка

BenchmarkOneScript v0.1.0, OneScript v2.0.0.567-rc7-29039316, Microsoft Windows NT 10.0.20348.0 \
Intel Core i7-8700 CPU 3.20GHz (Max: 3.19GHz) (Coffee Lake), 1 CPU, 12 logical and 6 physical cores

| Method   |     Mean |   StdErr |   StdDev |      Min |   Median |     Max |  Op/s |
|----------|---------:|---------:|---------:|---------:|---------:|--------:|------:|
| Бенчмарк | 208.9 ms | 198.3 ms | 443.4 ms | 10.54 ms | 10.66 ms | 1.002 s | 4.786 |

// Легенда
Mean   : Арифметическое среднее всех измерений
StdErr : Стандартная ошибка всех измерений
StdDev : Стандартное отклонение всех измерений
Min    : Минимум
Median : Значение, разделяющее упорядоченные измерения на две равные части
Max    : Максимум
Op/s   : Операций в секунду
1 ms   : 1 Миллисекунда
1 s    : 1 Секунда
```
