# Обработчики событий

Для изоляции логики подготовки и завершения от измеряемого кода в тестах производительности предусмотрен механизм обработчиков событий. 
Используйте специальные аннотации, чтобы определить методы, которые будут выполняться в разных фазах жизненного цикла бенчмарков:

#### &ПередВсеми
Аннотация `&ПередВсеми` помечает метод, который выполняется **один раз** перед запуском всех бенчмарков. 
Этот метод предназначен для глобальной подготовки данных, инициализации ресурсов (например, подключение к базе, загрузка файлов) или настройки окружения.

#### &ПослеВсех
Метод с аннотацией `&ПослеВсех` вызывается **после завершения всех бенчмарков**. 
Он используется для финализации: освобождения ресурсов (закрытие соединений), записи итоговых отчетов или выполнения других действий.

#### &ПередКаждым
Аннотация `&ПередКаждым` указывает, что метод должен выполняться **перед каждым запуском бенчмарка**. 
Это позволяет подготовить контекст для отдельного теста — например, сбросить состояние объекта, сгенерировать тестовые данные или выполнить предварительные проверки.

#### &ПослеКаждого
Метод с аннотацией `&ПослеКаждого` вызывается **после каждого выполнения бенчмарка**. 
Он используется для постобработки: валидации результатов, сброса временных значений или освобождения ресурсов, которые создавались для конкретного теста.

### API

```bsl
// Регистрация обработчика из произвольного объекта через делегат
Конфигурация.ДобавитьОбработчикСобытия(
	СобытияБенчмарков.ПередВсеми,
	Новый Делегат(Объект, "Инициализация")
);

// Регистрация обработчика из класса бенчмарка
Конфигурация.ДобавитьОбработчикСобытия
	СобытияБенчмарков.ПослеВсех,
	"Завершение"
);

// Удаление всех обработчиков для события
Конфигурация.УдалитьОбработчикиСобытия(СобытияБенчмарков.ПередКаждым);
```

### Пример

```bsl
Перем ДвоичныеДанные;

&ПередВсеми
Процедура Инициализация(Контекст) Экспорт
	ДвоичныеДанные = ПолучитьДвоичныеДанныеИзСтроки("Много данных");
КонецПроцедуры

&ПослеВсех
Процедура Завершение(Контекст) Экспорт
	ДвоичныеДанные = Неопределено;
КонецПроцедуры

&Бенчмарк
Процедура SHA256() Экспорт
	ХешированиеДанных = Новый ХешированиеДанных(ХешФункция.SHA256);
	ХешированиеДанных.Добавить(ДвоичныеДанные);
	ХешСумма = ХешированиеДанных.ХешСумма;
КонецПроцедуры

&Бенчмарк
Процедура MD5() Экспорт
	ХешированиеДанных = Новый ХешированиеДанных(ХешФункция.MD5);
	ХешированиеДанных.Добавить(ДвоичныеДанные);
	ХешСумма = ХешированиеДанных.ХешСумма;
КонецПроцедуры
```
